#!/usr/bin/perl -w

srand 1234;

use Geo::E00;
use Digest::MD5 qw{md5_hex};

my @lab;
open LAB, "data/lab";
for(<LAB>)
{
    /(\d+) (.*)/;
    $lab[$1]=$2;
}
close LAB;

my %oversea;
open OVS, "data/oversea";
for(<OVS>)
{
    /(.*?)\t+(.*)/;
    $oversea{$1}=$2;
}
close OVS;


my $e = new Geo::E00;
$e->open("data/wdbtemp4.e00") or die "can't open file";
my $edata = $e->parse;

my %pn;

for (@{$edata->{arc}}) {
    $left  = $lab[$_->{'poly-left'}] || "?";
    $right = $lab[$_->{'poly-right'}] || "?";
    $left  = $oversea{$left}  || $left;
    $right = $oversea{$right} || $right;
    $pn{$left}  ||= {} unless $left  eq "?";
    $pn{$right} ||= {} unless $right eq "?";
    next if ($left eq "?" or $right eq "?");
    $pn{$left}{$right} = 1;
    $pn{$right}{$left} = 1;
}

open WBD, "data/virtual_borders";
for (<WBD>)
{
    /(.*?)\t+(.*)/;
    $left  = $1;
    $right = $2;
    $left  = $oversea{$left}  || $left;
    $right = $oversea{$right} || $right;
    $pn{$left}{$right} = 1;
    $pn{$right}{$left} = 1;
}
close WBD;

%colors = ();

sub recursive_colorization
{
    my ($lev,$cntcolors,$alr) = @_;

    my %alln_layers = %$alr;
    my %this_layer  = ();
    my %next_layer  = ();

    for my $poly(keys %alln_layers)
    {
	$this_layer{$poly} = 1 if ((scalar keys %{$alln_layers{$poly}}) < $cntcolors);
    }
    for my $poly(keys %alln_layers)
    {
	next if $this_layer{$poly};
	$next_layer{$poly} ||= {};
	for my $p2 (keys %{$alln_layers{$poly}})
	{
	    $next_layer{$poly}{$p2} = 1 unless $this_layer{$p2}
	}
    }
    unless ((scalar keys %next_layer) == 0)
    {
	recursive_colorization($lev+1,$cntcolors,\%next_layer);
    }
    for my $poly(keys %this_layer)
    {
	my @cavail = (0..($cntcolors-1));
	my @n = keys %{$alln_layers{$poly}};
	for my $np (@n)
	{
	    my $c = $colors{$np};
	    $cavail[$c] = undef if defined $c;
	}
	my @pcolors;
	for my $cn (0..$#cavail)
	{
	    push @pcolors, $cn if (defined $cavail[$cn])
	}
	$colors{$poly} = $pcolors[rand @pcolors];
	die "Color error" unless defined $colors{$poly};
    }
}

recursive_colorization(0, 6, \%pn);

for $poly(keys %colors)
{
    my $m = $colors{$poly};
    $l = 0.8 + rand 0.2;
    $s = 0.1 + rand 0.4;

    if ($m == 0) {
	$r = 1;
	$g = 1-0.5*$s;
	$b = 1-$s;
    } elsif ($m == 1) {
	$r = 1;
	$g = 1;
	$b = 1-$s;
    } elsif ($m == 2) {
	$r = 1-0.5*$s;
	$g = 1;
	$b = 1-$s;
    } elsif ($m == 3) {
	$r = 1-$s;
	$g = 1;
	$b = 1-0.5*$s;
    } elsif ($m == 4) {
	$r = 1-0.5*$s;
	$g = 1-$s;
	$b = 1;
    } elsif ($m == 5) {
	$r = 1;
	$g = 1-$s;
	$b = 1;
    } else {
	die "Unknow color $m"
    }
    $r *= $l * 255;
    $g *= $l * 255;
    $b *= $l * 255;
    $colors{$poly} = sprintf "#%02X%02X%02X", $r, $g, $b;
}

my %cor;
open COR, "data/color_override";
for(<COR>)
{
    /(.*?)\t+(.*)/;
    $cor{$1} = $2;
}

open LAB, "data/lab";
for(<LAB>)
{
    /(\d+) (.*)/;
    if ($cor{$2}) {
	$color = $cor{$2}
    } elsif ($2 eq '?') {
	$color = "#E8E8FF"
    } else {
        $cnam = $oversea{$2} || $2;
        $color = $colors{$cnam};
    }
    die "$1 $2" unless $color;
    print "$1 $color\n"
}
close LAB;
